@@ -21,7 +21,7 @@ to communicate with a puppet infrastructure.
 
 USAGE
 -----
-puppet ssl <action> [-h|--help] [-v|--verbose] [-d|--debug] [--localca]
+puppet ssl <action> [-h|--help] [-v|--verbose] [-d|--debug] [--localca] [--target CERTNAME]
 
 
 OPTIONS
@@ -39,6 +39,8 @@ OPTIONS
 * --localca
   Also clean the local CA certificate and CRL.
 
+* --target CERTNAME
+  Clean the specified device certificate instead of this host's certificate.
 
 ACTIONS
 -------
@@ -62,10 +64,14 @@ ACTIONS
 * clean:
   Remove the private key and certificate related files for this host. If
   `--localca` is specified, then also remove this host's local copy of the
-  CA certificate(s) and CRL bundle.
+  CA certificate(s) and CRL bundle. if `--target CERTNAME` is specified, then
+  remove the files for the specified device on this host instead of this host.
 HELP
   end
 
+  option('--target CERTNAME') do |arg|
+    options[:target] = arg.to_s
+  end
   option('--localca')
   option('--verbose', '-v')
   option('--debug', '-d')
@@ -80,8 +86,17 @@ HELP
       raise Puppet::Error, _("An action must be specified.")
     end
 
-    Puppet.settings.use(:main, :agent)
-    host = Puppet::SSL::Host.new(options[:certname])
+    if options[:target]
+      # Override the following, as per lib/puppet/application/device.rb
+      Puppet[:certname] = options[:target]
+      Puppet[:confdir]  = File.join(Puppet[:devicedir], Puppet[:certname])
+      Puppet[:vardir]   = File.join(Puppet[:devicedir], Puppet[:certname])
+      host = Puppet::SSL::Host.new(Puppet[:certname], true)
+      Puppet.settings.use(:main, :agent, :device)
+    else
+      host = Puppet::SSL::Host.new(Puppet[:certname])
+      Puppet.settings.use(:main, :agent)
+    end
 
     action = command_line.args.first
     case action
@@ -166,16 +181,16 @@ HELP
 
   def clean(host)
     # make sure cert has been removed from the CA
-    if Puppet[:certname] == Puppet[:ca_server]
+    if host.name == Puppet[:ca_server]
       cert =
         begin
-          host.download_certificate_from_ca(Puppet[:certname])
+          host.download_certificate_from_ca(host.name)
         rescue => e
-          raise Puppet::Error.new(_("Failed to connect to the CA to determine if certificate %{certname} has been cleaned") % { certname: Puppet[:certname] }, e)
+          raise Puppet::Error.new(_("Failed to connect to the CA to determine if certificate %{certname} has been cleaned") % { certname: host.name }, e)
         end
 
       if cert
-        raise Puppet::Error, _(<<END) % { certname: Puppet[:certname] }
+        raise Puppet::Error, _(<<END) % { certname: host.name }
 The certificate %{certname} must be cleaned from the CA first. To fix this,
 run the following commands on the CA:
   puppetserver ca clean --certname %{certname}
@@ -184,16 +199,15 @@ END
       end
     end
 
-    settings = {
-      hostprivkey: 'private key',
-      hostpubkey: 'public key',
-      hostcsr: 'certificate request',
-      hostcert: 'certificate',
-      passfile: 'private key password file'
+    paths = {
+      'private key' => Puppet[:hostprivkey],
+      'public key'  => Puppet[:hostpubkey],
+      'certificate request' => File.join(Puppet[:requestdir], "#{Puppet[:certname]}.pem"),
+      'certificate' => Puppet[:hostcert],
+      'private key password file' => Puppet[:passfile]
     }
-    settings.merge!(localcacert: 'local CA certificate', hostcrl: 'local CRL') if options[:localca]
-    settings.each_pair do |setting, label|
-      path = Puppet[setting]
+    paths.merge!('local CA certificate' => Puppet[:localcacert], 'local CRL' => Puppet[:hostcrl]) if options[:localca]
+    paths.each_pair do |label, path|
       if Puppet::FileSystem.exist?(path)
         Puppet::FileSystem.unlink(path)
         Puppet.notice _("Removed %{label} %{path}") % { label: label, path: path }
