@@ -56,12 +56,8 @@ Puppet::Face.define(:module, '1.0.0') do
       EOT
     end
 
-    option '--strict-semver' do
-      summary _('Whether version ranges should exclude pre-release versions')
-    end
-
     when_invoked do |name, options|
-      name = name.tr('/', '-')
+      name = name.gsub('/', '-')
 
       Puppet::ModuleTool.set_option_defaults options
       message = if options[:version]
