@@ -18,27 +18,27 @@ class Puppet::FileServing::Metadata < Puppet::FileServing::Base
   PARAM_ORDER = [:mode, :ftype, :owner, :group]
 
   def checksum_type=(type)
-    raise(ArgumentError, "Unsupported checksum type #{type}") unless Puppet::Util::Checksums.respond_to?("#{type}_file")
+    raise(ArgumentError, _("Unsupported checksum type %{type}") % { type: type }) unless Puppet::Util::Checksums.respond_to?("#{type}_file")
 
     @checksum_type = type
   end
 
   def source_permissions=(source_permissions)
-    raise(ArgumentError, "Unsupported source_permission #{source_permissions}") unless [:use, :use_when_creating, :ignore].include?(source_permissions.intern)
+    raise(ArgumentError, _("Unsupported source_permission %{source_permissions}") % { source_permissions: source_permissions }) unless [:use, :use_when_creating, :ignore].include?(source_permissions.intern)
 
     @source_permissions = source_permissions.intern
   end
 
   def content_uri=(path)
     begin
-      uri = URI.parse(URI.escape(path))
+      uri = URI.parse(Puppet::Util.uri_encode(path))
     rescue URI::InvalidURIError => detail
-      raise(ArgumentError, "Could not understand URI #{path}: #{detail}")
+      raise(ArgumentError, _("Could not understand URI %{path}: %{detail}") % { path: path, detail: detail })
     end
-    raise(ArgumentError, "Cannot use opaque URLs '#{path}'") unless uri.hierarchical?
-    raise(ArgumentError, "Must use URLs of type puppet as content URI") if uri.scheme != "puppet"
+    raise(ArgumentError, _("Cannot use opaque URLs '%{path}'") % { path: path }) unless uri.hierarchical?
+    raise(ArgumentError, _("Must use URLs of type puppet as content URI")) if uri.scheme != "puppet"
 
-    @content_uri = path
+    @content_uri = path.encode(Encoding::UTF_8)
   end
 
   class MetaStat
@@ -65,14 +65,14 @@ class Puppet::FileServing::Metadata < Puppet::FileServing::Base
   end
 
   class WindowsStat < MetaStat
-    if Puppet.features.microsoft_windows?
+    if Puppet::Util::Platform.windows?
       require 'puppet/util/windows/security'
     end
 
     def initialize(stat, path, source_permissions)
       super(stat, source_permissions)
       @path = path
-      raise(ArgumentError, "Unsupported Windows source permissions option #{source_permissions}") unless @source_permissions_ignore
+      raise(ArgumentError, _("Unsupported Windows source permissions option %{source_permissions}") % { source_permissions: source_permissions }) unless @source_permissions_ignore
     end
 
     { :owner => 'S-1-5-32-544',
@@ -88,7 +88,7 @@ class Puppet::FileServing::Metadata < Puppet::FileServing::Base
   def collect_stat(path)
     stat = stat()
 
-    if Puppet.features.microsoft_windows?
+    if Puppet::Util::Platform.windows?
       WindowsStat.new(stat, path, @source_permissions)
     else
       MetaStat.new(stat, @source_permissions)
@@ -119,7 +119,7 @@ class Puppet::FileServing::Metadata < Puppet::FileServing::Base
       @destination = Puppet::FileSystem.readlink(real_path)
       @checksum = ("{#{@checksum_type}}") + send("#{@checksum_type}_file", real_path).to_s rescue nil
     else
-      raise ArgumentError, "Cannot manage files of type #{stat.ftype}"
+      raise ArgumentError, _("Cannot manage files of type %{file_type}") % { file_type: stat.ftype }
     end
   end
 
@@ -136,7 +136,8 @@ class Puppet::FileServing::Metadata < Puppet::FileServing::Base
     @destination = data.delete('destination')
     @source      = data.delete('source')
     @content_uri = data.delete('content_uri')
-    super(path,data)
+
+    super(path, data.map { |k, v| [k.to_sym, v] }.to_h)
   end
 
   def to_data_hash
