@@ -1,6 +1,5 @@
 require 'puppet/util/docs'
 require 'puppet/util/profiler'
-require 'puppet/util/methodhelper'
 require 'puppet/indirector/envelope'
 require 'puppet/indirector/request'
 
@@ -8,7 +7,6 @@ require 'puppet/indirector/request'
 # back-ends.  Each indirection has a set of associated terminus classes,
 # each of which is a subclass of Puppet::Indirector::Terminus.
 class Puppet::Indirector::Indirection
-  include Puppet::Util::MethodHelper
   include Puppet::Util::Docs
 
   attr_accessor :name, :model
@@ -89,25 +87,23 @@ class Puppet::Indirector::Indirection
     text
   end
 
-  def initialize(model, name, options = {})
+  def initialize(model, name, doc: nil, indirected_class: nil, cache_class: nil, terminus_class: nil, terminus_setting: nil, extend: nil)
     @model = model
     @name = name
     @termini = {}
 
-    @cache_class = nil
-    @terminus_class = nil
+    @doc = doc
 
     raise(ArgumentError, _("Indirection %{name} is already defined") % { name: @name }) if @@indirections.find { |i| i.name == @name }
     @@indirections << self
 
-    @indirected_class = options.delete(:indirected_class)
-    if mod = options[:extend]
-      extend(mod)
-      options.delete(:extend)
-    end
+    @indirected_class = indirected_class
+    self.extend(extend) if extend
 
-    # This is currently only used for cache_class and terminus_class.
-    set_options(options)
+    # These setters depend on the indirection already being installed so they have to be at the end
+    self.cache_class = cache_class if cache_class
+    self.terminus_class = terminus_class if terminus_class
+    self.terminus_setting = terminus_setting if terminus_setting
   end
 
   # Set up our request object.
