@@ -10,8 +10,7 @@ Puppet::Type.type(:package).provide :pip,
   desc "Python packages via `pip`.
 
   This provider supports the `install_options` attribute, which allows command-line flags to be passed to pip.
-  These options should be specified as a string (e.g. '--flag'), a hash (e.g. {'--flag' => 'value'}),
-  or an array where each element is either a string or a hash."
+  These options should be specified as an array where each element is either a string or a hash."
 
   has_feature :installable, :uninstallable, :upgradeable, :versionable, :install_options
 
