@@ -25,8 +25,7 @@ class Puppet::Settings::ConfigFile
       allowed_section_names << 'main' unless allowed_section_names.include?('main')
     end
 
-    # in Ruby 1.9.3 strings are not UTF-8 by default, so ensure text is treated properly
-    ini = Puppet::Settings::IniFile.parse(StringIO.new(text).set_encoding(Encoding::UTF_8))
+    ini = Puppet::Settings::IniFile.parse(text.encode(Encoding::UTF_8))
     unique_sections_in(ini, file, allowed_section_names).each do |section_name|
       section = Section.new(section_name.to_sym)
       result.with_section(section)
@@ -90,7 +89,7 @@ private
         message += ' ' + _("The only valid puppet.conf sections are: [%{allowed_sections_list}].") %
             { allowed_sections_list: allowed_section_names.join(", ") }
         message += ' ' + _("Please use the directory environments feature to specify environments.")
-        message += ' ' + _("(See https://docs.puppet.com/puppet/latest/reference/environments.html)")
+        message += ' ' + _("(See https://puppet.com/docs/puppet/latest/environments_about.html)")
         raise(Puppet::Error, message)
       end
       section.name
