@@ -150,13 +150,9 @@ module Puppet
             end
           end
           unless log == :false
-            if @resource.parameter(:command).sensitive
-              self.send(log, "[output redacted]")
-            else
-              @output.split(/\n/).each { |line|
-                self.send(log, line)
-              }
-            end
+            @output.split(/\n/).each { |line|
+              self.send(log, line)
+            }
           end
         end
 
@@ -214,7 +210,7 @@ module Puppet
         this attribute."
 
       validate do |user|
-        if Puppet.features.microsoft_windows?
+        if Puppet::Util::Platform.windows?
           self.fail _("Unable to execute commands as other users on Windows")
         elsif !Puppet.features.root? && resource.current_username() != user
           self.fail _("Only root can execute commands as other users")
@@ -459,13 +455,9 @@ module Puppet
           return false
         end
 
-        if sensitive
-          self.debug("[output redacted]")
-        else
-          output.split(/\n/).each { |line|
-            self.debug(line)
-          }
-        end
+        output.split(/\n/).each { |line|
+          self.debug(line)
+        }
 
         status.exitstatus != 0
       end
@@ -515,13 +507,9 @@ module Puppet
           return false
         end
 
-        if sensitive
-          self.debug("[output redacted]")
-        else
-          output.split(/\n/).each { |line|
-            self.debug(line)
-          }
-        end
+        output.split(/\n/).each { |line|
+          self.debug(line)
+        }
 
         status.exitstatus == 0
       end
@@ -541,7 +529,7 @@ module Puppet
       # Stick the cwd in there if we have it
       reqs << self[:cwd] if self[:cwd]
 
-      file_regex = Puppet.features.microsoft_windows? ? %r{^([a-zA-Z]:[\\/]\S+)} : %r{^(/\S+)}
+      file_regex = Puppet::Util::Platform.windows? ? %r{^([a-zA-Z]:[\\/]\S+)} : %r{^(/\S+)}
 
       self[:command].scan(file_regex) { |str|
         reqs << str
@@ -593,15 +581,7 @@ module Puppet
           val = @parameters[check].value
           val = [val] unless val.is_a? Array
           val.each do |value|
-            if !@parameters[check].check(value)
-              # Give a debug message so users can figure out what command would have been
-              # but don't print sensitive commands or parameters in the clear
-              cmdstring = @parameters[:command].sensitive ? "[command redacted]" : @parameters[:command].value
-
-              debug(_("'%{cmd}' won't be executed because of failed check '%{check}'") % { cmd: cmdstring, check: check })
-
-              return false 
-            end
+            return false unless @parameters[check].check(value)
           end
         end
       }
@@ -634,25 +614,11 @@ module Puppet
 
     private
     def set_sensitive_parameters(sensitive_parameters)
-      # If any are sensitive, mark all as sensitive
-      sensitive = false
-      parameters_to_check = [:command, :unless, :onlyif]
-
-      parameters_to_check.each do |p| 
-        if sensitive_parameters.include?(p)
-          sensitive_parameters.delete(p)
-          sensitive = true
-        end
+      # Respect sensitive commands
+      if sensitive_parameters.include?(:command)
+        sensitive_parameters.delete(:command)
+        parameter(:command).sensitive = true
       end
-
-      if sensitive
-        parameters_to_check.each do |p|
-          if parameters.include?(p)
-            parameter(p).sensitive = true
-          end
-        end
-      end
-
       super(sensitive_parameters)
     end
   end
