@@ -42,23 +42,19 @@ module Puppet
     end
 
     newparam(:server) do
-      desc "The server providing the remote filebucket service.
+      desc "The server providing the remote filebucket service. Defaults to the
+        value of the `server` setting (that is, the currently configured
+        puppet master server).
 
-        This setting is _only_ consulted if the `path` attribute is set to `false`.
-
-        If this attribute is not specified, the first entry in the `server_list`
-        configuration setting is used, followed by the value of the `server` setting
-        if `server_list` is not set."
+        This setting is _only_ consulted if the `path` attribute is set to `false`."
+      defaultto { Puppet[:server] }
     end
 
     newparam(:port) do
-      desc "The port on which the remote server is listening.
+      desc "The port on which the remote server is listening. Defaults to the
+        value of the `masterport` setting, which is usually %s." % Puppet[:masterport]
 
-        This setting is _only_ consulted if the `path` attribute is set to `false`.
-
-        If this attribute is not specified, the first entry in the `server_list`
-        configuration setting is used, followed by the value of the `masterport`
-        setting if `server_list` is not set."
+      defaultto { Puppet[:masterport] }
     end
 
     newparam(:path) do
