@@ -22,13 +22,12 @@ module Util
   require 'puppet/util/posix'
   extend Puppet::Util::POSIX
 
-  # Can't use Puppet.features.microsoft_windows? as it may be mocked out in a test.  This can cause test recurring test failures
   require 'puppet/util/windows/process' if Puppet::Util::Platform.windows?
 
   extend Puppet::Util::SymbolicFileMode
 
   def default_env
-    Puppet.features.microsoft_windows? ?
+    Puppet::Util::Platform.windows? ?
       :windows :
       :posix
   end
@@ -272,7 +271,7 @@ module Util
             raise
           end
         else
-          if Puppet.features.microsoft_windows? && File.extname(dest).empty?
+          if Puppet::Util::Platform.windows? && File.extname(dest).empty?
             exts.each do |ext|
               destext = File.expand_path(dest + ext)
               return destext if FileTest.file? destext and FileTest.executable? destext
@@ -322,8 +321,8 @@ module Util
 
     params = { :scheme => 'file' }
 
-    if Puppet.features.microsoft_windows?
-      path = path.tr('\\', '/')
+    if Puppet::Util::Platform.windows?
+      path = path.gsub(/\\/, '/')
 
       if unc = /^\/\/([^\/]+)(\/.+)/.match(path)
         params[:host] = unc[1]
@@ -356,7 +355,7 @@ module Util
     # URI.unescape does, but returns strings in their original encoding
     path = URI.unescape(uri.path.encode(Encoding::UTF_8))
 
-    if Puppet.features.microsoft_windows? and uri.scheme == 'file'
+    if Puppet::Util::Platform.windows? && uri.scheme == 'file'
       if uri.host
         path = "//#{uri.host}" + path # UNC
       else
@@ -524,44 +523,21 @@ module Util
 
   module_function :thinmark
 
-  PUPPET_STACK_INSERTION_FRAME = /.*puppet_stack\.rb.*in.*`stack'/
-
   # utility method to get the current call stack and format it to a human-readable string (which some IDEs/editors
   # will recognize as links to the line numbers in the trace)
-  def self.pretty_backtrace(backtrace = caller(1), puppetstack = [])
-    format_backtrace_array(backtrace, puppetstack).join("\n")
-  end
-
-  # arguments may be a Ruby stack, with an optional Puppet stack argument,
-  # or just a Puppet stack.
-  # stacks may be an Array of Strings "/foo.rb:0 in `blah'" or
-  # an Array of Arrays that represent a frame: ["/foo.pp", 0]
-  def self.format_backtrace_array(primary_stack, puppetstack = [])
-    primary_stack.flat_map do |frame|
-      frame = format_puppetstack_frame(frame) if frame.is_a?(Array)
-      primary_frame = resolve_stackframe(frame)
-
-      if primary_frame =~ PUPPET_STACK_INSERTION_FRAME && !puppetstack.empty?
-        [resolve_stackframe(format_puppetstack_frame(puppetstack.shift)),
-         primary_frame]
+  def self.pretty_backtrace(backtrace = caller(1))
+    backtrace.collect do |line|
+      _, path, rest = /^(.*):(\d+.*)$/.match(line).to_a
+      # If the path doesn't exist - like in one test, and like could happen in
+      # the world - we should just tolerate it and carry on. --daniel 2012-09-05
+      # Also, if we don't match, just include the whole line.
+      if path
+        path = Pathname(path).realpath rescue path
+        "#{path}:#{rest}"
       else
-        primary_frame
+        line
       end
-    end
-  end
-
-  def self.resolve_stackframe(frame)
-    _, path, rest = /^(.*):(\d+.*)$/.match(frame).to_a
-    if path
-      path = Pathname(path).realpath rescue path
-      "#{path}:#{rest}"
-    else
-      frame
-    end
-  end
-
-  def self.format_puppetstack_frame(file_and_lineno)
-    file_and_lineno.join(':')
+    end.join("\n")
   end
 
   # Replace a file, securely.  This takes a block, and passes it the file
@@ -597,7 +573,7 @@ module Util
 
       mode = symbolic_mode_to_int(normalize_symbolic_mode(default_mode))
     else
-      if Puppet.features.microsoft_windows?
+      if Puppet::Util::Platform.windows?
         mode = DEFAULT_WINDOWS_MODE
       else
         mode = DEFAULT_POSIX_MODE
@@ -610,9 +586,9 @@ module Util
       tempfile = Puppet::FileSystem::Uniquefile.new(Puppet::FileSystem.basename_string(file), Puppet::FileSystem.dir_string(file))
 
       effective_mode =
-      if !Puppet.features.microsoft_windows?
+      if !Puppet::Util::Platform.windows?
         # Grab the current file mode, and fall back to the defaults.
-        
+
         if Puppet::FileSystem.exist?(file)
           stat = Puppet::FileSystem.lstat(file)
           tempfile.chown(stat.uid, stat.gid)
@@ -647,7 +623,7 @@ module Util
 
       tempfile.close
 
-      if Puppet.features.microsoft_windows?
+      if Puppet::Util::Platform.windows?
         # Windows ReplaceFile needs a file to exist, so touch handles this
         if !Puppet::FileSystem.exist?(file)
           Puppet::FileSystem.touch(file)
@@ -660,6 +636,11 @@ module Util
         Puppet::Util::Windows::File.replace_file(FileSystem.path_string(file), tempfile.path)
 
       else
+        # MRI Ruby checks for this and raises an error, while JRuby removes the directory
+        # and replaces it with a file. This makes the our version of replace_file() consistent
+        if Puppet::FileSystem.exist?(file) && Puppet::FileSystem.directory?(file)
+          raise Errno::EISDIR, _("Is a directory: %{directory}") % { directory: file }
+        end
         File.rename(tempfile.path, Puppet::FileSystem.path_string(file))
       end
     ensure
@@ -719,7 +700,6 @@ end
 
 
 require 'puppet/util/errors'
-require 'puppet/util/methodhelper'
 require 'puppet/util/metaid'
 require 'puppet/util/classgen'
 require 'puppet/util/docs'
