@@ -13,7 +13,7 @@ require 'pathname'
 
 module Puppet::Util::SELinux
 
-  def self.selinux_support?
+  def selinux_support?
     return false unless defined?(Selinux)
     if Selinux.is_selinux_enabled == 1
       return true
@@ -21,10 +21,6 @@ module Puppet::Util::SELinux
     false
   end
 
-  def selinux_support?
-    Puppet::Util::SELinux.selinux_support?
-  end
-
   # Retrieve and return the full context of the file.  If we don't have
   # SELinux support or if the SELinux call fails then return nil.
   def get_selinux_current_context(file)
